==> makeen-expressionengine-0.0.51.tgz: package/src/functions/globals/date.ts <==
﻿import { mfunction } from '../mfunction';
import { List } from 'typescript-dotnet-commonjs/System/Collections/List'
import expressionData from '../../expressionData'
import { expressionPart } from './../../compiler/parts/expressionPart'
import { groupDataContext } from './../../functions/groupDataContext'
import { DateTime } from 'typescript-dotnet-commonjs/System/Time/DateTime'
import { TimeSpan } from 'typescript-dotnet-commonjs/System/Time/TimeSpan'
import { Type } from 'typescript-dotnet-commonjs/System/Types'
import { utils } from './../../compiler/utils'
import { outputVariables } from '../../utils/outputVariables';

export class isDate extends mfunction {

    public get minParameters(): number {
        return 1;
    }

    public get maxParameters(): number {
        return 2;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        resolved.resolved = false;
        let value: Object | undefined = undefined;
        let matchYear: boolean = true;
        if (parameters.count > 0) {
            value = (parameters.count > 0) ? (parameters.get(0).resolve(expressionData, /* out */resolved, /* running:*/ running) as string) : DateTime.now.toString();
            if ((parameters.count > 1)) {
                matchYear = parameters.get(1).resolve(expressionData, /* out */resolved, /* running:*/ running) as boolean;
            }
        }
        else {
            resolved.resolved = false;
        }

        let paramDate: DateTime | undefined = undefined;
        if (resolved.resolved && obj != null && (value instanceof DateTime || typeof value == "string")) {
            let paramIsDate: boolean = false;
            if (value instanceof DateTime) {
                paramDate = (<DateTime>(value));
                paramIsDate = true;
            }
            else {
                paramDate = new DateTime(Date.parse(value?.toString() as string));
            }

            return (paramIsDate && utils.matchDates(paramDate as DateTime, (<DateTime>(obj)), matchYear));
        }

        resolved.resolved = false;
        return false;
    }
}

export class isToday extends mfunction {

    public get minParameters(): number {
        return 0;
    }

    public get maxParameters(): number {
        return 0;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        resolved.resolved = true;
        let matchYear: boolean = true;
        if (parameters.count > 0) {
            matchYear = parameters.get(0).resolve(expressionData, /* out */resolved, /* running:*/ running) as boolean;
        }

        return (obj instanceof DateTime && utils.matchDates(DateTime.now, (<DateTime>(obj)), matchYear));
    }
}

export class now extends mfunction {

    public get minParameters(): number {
        return 0;
    }

    public get maxParameters(): number {
        return 0;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        resolved.resolved = true;
        return Date.parse(DateTime.now.toJsDate().toString());
    }
}

export class utcNow extends mfunction {

    public get minParameters(): number {
        return 0;
    }

    public get maxParameters(): number {
        return 0;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        resolved.resolved = true;
        return Date.parse(DateTime.now.toUniversalTime().toJsDate().toString());
    }
}

export class addPeriod extends mfunction {

    public get minParameters(): number {
        return 1;
    }

    public get maxParameters(): number {
        return 2;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        let subResolved: outputVariables = new outputVariables();
        let val: number = parameters.count > 0 ? parameters.get(0).resolve(expressionData, /* out */subResolved) as number : 0;
        let type: string = parameters.count > 1 ? parameters.get(1).resolve(expressionData, /* out */subResolved) as string : "";

        let addition: TimeSpan = TimeSpan.fromTicks(0);
        if ((obj != null)) {
            switch (type.toLowerCase()) {
                case "day":
                case "days":
                    addition = TimeSpan.fromDays(val);
                    break;
                case "week":
                case "weeks":
                    addition = TimeSpan.fromDays((val * 7));
                    break;
                case "month":
                case "months":
                    if (obj instanceof DateTime || obj instanceof DateTime) {
                        obj = (<DateTime>(obj)).addMonths((<number>(val)));
                    }
                    else {
                        addition = TimeSpan.fromDays((30 * val));
                    }

                    break;
                case "yr":
                case "yrs":
                case "year":
                case "years":
                    if (obj instanceof DateTime || obj instanceof DateTime) {
                        obj = (<DateTime>(obj)).addYears((<number>(val)));
                    }
                    else {
                        addition = TimeSpan.fromDays((365 * val));
                    }

                    break;
                case "hr":
                case "hrs":
                case "hour":
                case "hours":
                    addition = TimeSpan.fromHours(val);
                    break;
                case "min":
                case "mins":
                case "minute":
                case "minutes":
                    addition = TimeSpan.fromMinutes(val);
                    break;
                case "sec":
                case "secs":
                case "second":
                case "seconds":
                    addition = TimeSpan.fromSeconds(val);
                    break;
                case "ms":
                case "millis":
                case "millisec":
                case "millisecs":
                case "millisecond":
                case "milliseconds":
                    addition = TimeSpan.fromMilliseconds(val);
                    break;
                case "tick":
                case "ticks":
                    break;
                default:
                    addition = TimeSpan.fromTicks(val);
                    break;
            }

            if (obj instanceof DateTime || obj instanceof DateTime) {
                obj = (<DateTime>(obj)).add(addition);
                resolved.resolved = true;
            }
            else if (obj instanceof TimeSpan || obj instanceof TimeSpan) {
                obj = (<TimeSpan>(obj)).add(addition);
                resolved.resolved = true;
            }
            else {
                resolved.resolved = false;
            }

        }
        else {
            resolved.resolved = false;
        }

        return obj;
    }
}

export class toUtc extends mfunction {

    public get minParameters(): number {
        return 0;
    }

    public get maxParameters(): number {
        return 0;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        if (obj instanceof DateTime || obj instanceof DateTime) {
            obj = (<DateTime>(obj)).toUniversalTime();
            resolved.resolved = true;
        }
        else {
            resolved.resolved = false;
        }

        return obj;
    }
}

export class toLocalTime extends mfunction {

    public get minParameters(): number {
        return 0;
    }

    public get maxParameters(): number {
        return 0;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        if (obj instanceof Date || obj instanceof Date) {
            obj = (<Date>(obj)).toLocaleTimeString();
            resolved.resolved = true;
        }
        else if (obj instanceof DateTime || obj instanceof DateTime) {
            obj = (<DateTime>(obj)).toJsDate().toLocaleTimeString();
            resolved.resolved = true;
        }
        else {
            resolved.resolved = false;
        }

        return obj;
    }
}

==> makeen-expressionengine-0.0.63.tgz: package/src/functions/globals/date.ts <==
﻿import { mfunction } from '../mfunction';
import expressionData from '../../expressionData'
import { expressionPart } from './../../compiler/parts/expressionPart'
import { groupDataContext } from './../../functions/groupDataContext'
import { DateTime } from 'typescript-dotnet-commonjs/System/Time/DateTime'
import { TimeSpan } from 'typescript-dotnet-commonjs/System/Time/TimeSpan'
import { utils } from './../../compiler/utils'
import { outputVariables } from '../../utils/outputVariables';
// import { lo } from 'date-fns';

export class isDate extends mfunction {

     get minParameters(): number {
        return 1;
    }

     get maxParameters(): number {
        return 2;
    }

     invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        resolved.resolved = false;
        let value: Object | undefined = undefined;
        let matchYear: boolean = true;
        if (parameters.length > 0) {
            value = (parameters.length > 0) ? (parameters[0].resolve(expressionData, /* out */resolved, /* running:*/ running) as string) : DateTime.now.toString();
            if ((parameters.length > 1)) {
                matchYear = parameters[1].resolve(expressionData, /* out */resolved, /* running:*/ running) as boolean;
            }
        }
        else {
            resolved.resolved = false;
        }

        let paramDate: DateTime | undefined = undefined;
        if (resolved.resolved && obj != null && (value instanceof DateTime || typeof value == "string")) {
            let paramIsDate: boolean = false;
            if (value instanceof DateTime) {
                paramDate = (<DateTime>(value));
                paramIsDate = true;
            }
            else {
                paramDate = new DateTime(Date.parse(value?.toString() as string));
            }

            return (paramIsDate && utils.matchDates(paramDate as DateTime, (<DateTime>(obj)), matchYear));
        }

        resolved.resolved = false;
        return false;
    }
}

export class isToday extends mfunction {

     get minParameters(): number {
        return 0;
    }

     get maxParameters(): number {
        return 0;
    }

     invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        resolved.resolved = true;
        let matchYear: boolean = true;
        if (parameters.length > 0) {
            matchYear = parameters[0].resolve(expressionData, /* out */resolved, /* running:*/ running) as boolean;
        }

        return (obj instanceof DateTime && utils.matchDates(DateTime.now, (<DateTime>(obj)), matchYear));
    }
}

export class now extends mfunction {

     get minParameters(): number {
        return 0;
    }

     get maxParameters(): number {
        return 0;
    }

     invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        resolved.resolved = true;
        return Date.parse(DateTime.now.toJsDate().toString());
    }
}

export class utcNow extends mfunction {

     get minParameters(): number {
        return 0;
    }

     get maxParameters(): number {
        return 0;
    }

     invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        resolved.resolved = true;
        return Date.parse(DateTime.now.toUniversalTime().toJsDate().toString());
    }
}

export class addPeriod extends mfunction {

     get minParameters(): number {
        return 1;
    }

     get maxParameters(): number {
        return 2;
    }

     invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        let subResolved: outputVariables = new outputVariables();
        let val: number = parameters.length > 0 ? parameters[0].resolve(expressionData, /* out */subResolved) as number : 0;
        let type: string = parameters.length > 1 ? parameters[1].resolve(expressionData, /* out */subResolved) as string : "";

        let addition: TimeSpan = TimeSpan.fromTicks(0);
        if ((obj != null)) {
            switch (type.toLowerCase()) {
                case "day":
                case "days":
                    addition = TimeSpan.fromDays(val);
                    break;
                case "week":
                case "weeks":
                    addition = TimeSpan.fromDays((val * 7));
                    break;
                case "month":
                case "months":
                    if (obj instanceof DateTime || obj instanceof DateTime) {
                        obj = (<DateTime>(obj)).addMonths((<number>(val)));
                    }
                    else {
                        addition = TimeSpan.fromDays((30 * val));
                    }

                    break;
                case "yr":
                case "yrs":
                case "year":
                case "years":
                    if (obj instanceof DateTime || obj instanceof DateTime) {
                        obj = (<DateTime>(obj)).addYears((<number>(val)));
                    }
                    else {
                        addition = TimeSpan.fromDays((365 * val));
                    }

                    break;
                case "hr":
                case "hrs":
                case "hour":
                case "hours":
                    addition = TimeSpan.fromHours(val);
                    break;
                case "min":
                case "mins":
                case "minute":
                case "minutes":
                    addition = TimeSpan.fromMinutes(val);
                    break;
                case "sec":
                case "secs":
                case "second":
                case "seconds":
                    addition = TimeSpan.fromSeconds(val);
                    break;
                case "ms":
                case "millis":
                case "millisec":
                case "millisecs":
                case "millisecond":
                case "milliseconds":
                    addition = TimeSpan.fromMilliseconds(val);
                    break;
                case "tick":
                case "ticks":
                    break;
                default:
                    addition = TimeSpan.fromTicks(val);
                    break;
            }

            if (obj instanceof DateTime || obj instanceof DateTime) {
                obj = (<DateTime>(obj)).add(addition);
                resolved.resolved = true;
            }
            else if (obj instanceof TimeSpan || obj instanceof TimeSpan) {
                obj = (<TimeSpan>(obj)).add(addition);
                resolved.resolved = true;
            }
            else {
                resolved.resolved = false;
            }

        }
        else {
            resolved.resolved = false;
        }

        return obj;
    }
}

export class toUtc extends mfunction {

     get minParameters(): number {
        return 0;
    }

     get maxParameters(): number {
        return 0;
    }

     invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        if (obj instanceof DateTime || obj instanceof DateTime) {
            obj = (<DateTime>(obj)).toUniversalTime();
            resolved.resolved = true;
        }
        else {
            resolved.resolved = false;
        }

        return obj;
    }
}

export class toLocalTime extends mfunction {

     get minParameters(): number {
        return 0;
    }

     get maxParameters(): number {
        return 0;
    }

     invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        if (obj instanceof Date || obj instanceof Date) {
            obj = (<Date>(obj)).toLocaleString();
            resolved.resolved = true;
        }
        else if (obj instanceof DateTime || obj instanceof DateTime) {
            obj = (<DateTime>(obj)).toJsDate().toLocaleString();
            resolved.resolved = true;
        }
        else if(typeof obj==='number' || typeof obj==='string'){
            const dt = new Date(obj);
            resolved.resolved = true;
            return dt.toLocaleString();
        }
        else {
            resolved.resolved = false;
        }

        return obj;
    }
}
